package pt.uc.dei.wsvdbench.tpcapp.validator;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import org.apache.commons.validator.EmailValidator;
import org.apache.commons.validator.ValidatorException;
import pt.uc.dei.wsdbench.generic.component.pojo.Country;
import pt.uc.dei.wsdbench.generic.component.spring.BeanFactory;
import pt.uc.dei.wsdbench.generic.component.tpcapp.Constants;
import pt.uc.dei.wsdbench.sut.configuration.Configurator;


import pt.uc.dei.wsdbench.loader.configuration.CountryReader;
import pt.uc.dei.wsvdbench.tpcapp.input.ChangePaymentMethodInput;
import pt.uc.dei.wsvdbench.tpcapp.input.NewCustomerInput;
import pt.uc.dei.wsvdbench.tpcapp.input.NewProductsInput;
import pt.uc.dei.wsvdbench.tpcapp.input.ProductDetailInput;

/**
 * WS - Vulnerability Detection Tools Benchmark
 * TPC - APP Benchmark Services
 *
 *
 *
 *
 * @author cnl@dei.uc.pt
 * @author nmsa@dei.uc.pt
 */
public class InputValidator {

    private Configurator configurator;
    private CountryReader countryReader;

    public void setConfigurator(Configurator configurator) {
        this.configurator = configurator;
    }

    public void setCountryReader(CountryReader countryReader) {
        this.countryReader = countryReader;
    }
    //
    //
    private static InputValidator inputValidatorBean;

    public static InputValidator getInstance() {
//        System.out.println("getInstance ");
        if (inputValidatorBean == null) {
//            System.out.println("validatorsBean == null ");
            try {
                inputValidatorBean = (InputValidator) BeanFactory.getBean("inputValidatorBean");
                System.out.println("InputValidator CREATED: " + inputValidatorBean);
            } catch (Exception e) {
                System.out.println("Exception in inputValidatorBean = (InputValidator) BeanFactory.getBean(\"inputValidatorBean\");");
                e.printStackTrace();
            }
        }
//        System.out.println("validatorsBean\t\t\t\t: " + validatorsBean);
//        System.out.println("configurator  \t\t\t\t: " + validatorsBean.configurator);
//        System.out.println("configurator  \t\t\t\t: " + validatorsBean.configurator.isUseValidation());
        return inputValidatorBean;
    }

    public void validate(ProductDetailInput input) throws ValidatorException {
        validateFields(input);
        if (input.getItemIds() == null) {//NOT VULN
            throw new ValidatorException("Product detail ids list is currently null");
        } else if (input.getItemIds().size() > Constants.MAX_PRODUCT_DETAIL_IDS || input.getItemIds().size() == 0) {
            throw new ValidatorException("Product detail ids list size invalid. Asked for " + input.getItemIds().size() + " items. (Maximum is " + Constants.MAX_PRODUCT_DETAIL_IDS + ")");
        }
    }

    public void validate(NewProductsInput input) throws ValidatorException {
        validateFields(input);
        validateNumber(1, 10, "cutOffDuration", input.getCutOffDuration());//NOT VULN
        final List<String> subjects = Arrays.asList(Constants.SUBJECT_LIST);
        if (!subjects.contains(input.getSubjectString())) {//NOT VULN
            throw new ValidatorException("Subject (" + input.getSubjectString() + ") not found in subject list: " + Constants.SUBJECT_LIST);
        }
        validateNumber(1, new Long(configurator.getItemCount()), "itemLimit", input.getItemLimit());//NOT VULN
    }

    public void validate(ChangePaymentMethodInput input) throws ValidatorException {
        validateFields(input);
        this.validatePaymentMethod(input.getPaymentMethod());//NOT VULN
        this.validateAString(40, 300, "creditInfo", input.getCreditInfo());//VULN
        this.validateNString(1, 1000000, "poId", input.getPoId());//NOT VULN
    }

    public void validate(NewCustomerInput input) throws ValidatorException {
        validateFields(input);
        validateAString(15, 20, "businessName", input.getBusinessName());//VULN
        validateAString(40, 100, "businessInfo", input.getBusinessInfo());//VULN
        validatePassword(input);//VULN
        validateAString(8, 15, "contactFName", input.getContactFName());//VULN
        validateAString(8, 15, "contactLName", input.getContactLName());//VULN
        validateNString(9, 16, "contactPhone", input.getContactPhone());//NOT VULN
        validateEmailFormat(input);//VULN
        validatePaymentMethod(input.getPaymentMethod());//NOT VULN
        validateNumber(1, 1000000, "poId", input.getPoId());//NOT VULN
        validateAString(40, 300, "creditInfo", input.getCreditInfo());//VULN
        validateAString(15, 40, "billingAddr1", input.getBillingAddr1());//VULN
        validateAString(15, 40, "billingAddr2", input.getBillingAddr2());//VULN
        validateAString(4, 30, "billingCity", input.getBillingCity());//VULN
        validateAString(2, 20, "billingState", input.getBillingState());//VULN
        validateAString(5, 10, "billingZip", input.getBillingZip());//VULN
        validateCountry(input.getBillingCountry());//NOT VULN
    }
    ///
    ///
    ///
    ///
    ///

    public void validateFields(Object obj) throws ValidatorException {
        if (obj == null) {
            throw new ValidatorException("Null object supplied!");
        }
        Field[] fields = obj.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            try {
                Object value = field.get(obj);
                if (value == null) {
                    throw new ValidatorException("Null value was found for one of the supplied object fields. Object class is " + obj.getClass().getName());
                }
            } catch (IllegalAccessException e) {
                System.out.println("Exception:" + e);
            }
        }
    }

    public void validateCountry(String name) throws ValidatorException {
        boolean found = false;

        List<Country> list = countryReader.getCountries();
        for (int i = 0, n = countryReader.getCountries().size(); i < n; i++) {
            if (name.equals(list.get(i).getName())) {
                found = true;
                break;
            }
        }
        if (!found) {
            throw new ValidatorException("Country: " + name + "not found.");
        }
    }

    public void validateNString(int min, int max, String type, String toValidate) throws ValidatorException {
        boolean isNumber = false;
        isNumber = org.apache.commons.validator.GenericValidator.isLong(toValidate);

        if (toValidate.length() < min || toValidate.length() > max || !isNumber) {
            throw new ValidatorException("Invalid NString for " + type + " string: " + toValidate);
        }
    }

    public void validateAString(int min, int max, String type, String toValidate) throws ValidatorException {
        if (toValidate.length() < min || toValidate.length() > max) {
            throw new ValidatorException("Invalid length for " + type + " string: " + toValidate);
        }
    }

    public void validateNumber(long min, long max, String type, Long number)
            throws ValidatorException {
        if (number == null || number > max || number < min) {
            throw new ValidatorException("Invalid value for " + type + ". Value = " + number + ". Should be between " + min + " and " + max);
        }
    }

    public void validatePassword(NewCustomerInput input)
            throws ValidatorException {
        if (!input.getBusinessName().toLowerCase().equals(input.getPassword())) {
            throw new ValidatorException("Invalid password: " + input.getPassword() + " (businessName: " + input.getBusinessName());
        }
    }

    public void validatePaymentMethod(String method) throws ValidatorException {
        if (!method.equals("PO") && !method.equals("CC")) {
            throw new ValidatorException("Payment method is not PO or CC. paymentMethod: " + method);
        }
    }

    public void validateEmailFormat(NewCustomerInput input) throws ValidatorException {
        validateEmail(input.getContactEmail());
        if (!input.getContactEmail().equals(
                input.getContactFName() + "@" + input.getBusinessName() + ".com")) {
            throw new ValidatorException("Wrong email:" + input.getContactEmail() + ". Should be" + input.getContactFName() + "@" + input.getBusinessName() + ".com");
        }
    }

    private void validateEmail(String email) throws ValidatorException {
        boolean validEmail = false;

        if (email.length() > 50) {
            throw new ValidatorException(
                    "Invalid size for Email. Should be 50 chars max. Email: " + email);
        }

        validEmail = EmailValidator.getInstance().isValid(email);
        if (!validEmail) {
            throw new ValidatorException("Invalid Email: " + email);
        }
    }
}
