package pt.uc.dei.wsvdbench.tpcapp.versions;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import pt.uc.dei.wsvdbench.tpcapp.input.NewProductsInput;
import pt.uc.dei.wsvdbench.tpcapp.output.NewProduct;
import pt.uc.dei.wsvdbench.tpcapp.output.NewProductsOutput;
import pt.uc.dei.wsvdbench.util.Database;

/**
 * WS - Vulnerability Detection Tools Benchmark
 * TPC - APP Benchmark Services
 *
 *
 * #WebServiceOperation
 *
 * @author cnl@dei.uc.pt
 * @author nmsa@dei.uc.pt
 */
public class NewProducts_Vx0 {

    public NewProductsOutput process(NewProductsInput input) throws Exception {
        pt.uc.dei.wsvdbench.tpcapp.validator.InputValidator.getInstance().validate(input);
        NewProductsOutput npo = new NewProductsOutput();
        Connection conn = Database.pickConnection();
        try {
            if (conn != null) {
                npo = getData(conn, npo, input);
            }
            conn.commit();
        } catch (Exception e) {
            throw e;
        } finally {
            Database.relaseConnection(conn);
        }
        return npo;
    }

    private synchronized NewProductsOutput getData(Connection conn, NewProductsOutput npo, NewProductsInput input) throws SQLException {
        // setfetch size item_limit
        Long il = input.getItemLimit();
        Long l = null;
        PreparedStatement ps82 = conn.prepareStatement(
                "   SELECT i_id, i_title, a_fname, a_lname " +
                "     FROM item, author " +
                "    WHERE lower(i_subject) = lower(?) " +
                "      AND i_pub_date > to_timestamp(?, 'YYYY-MM-DD HH24:MI:SS') " +
                "      AND i_a_id = a_id  " +
                " ORDER BY i_pub_date desc, i_title asc");

        if (il != null) {
            l = new Long(il);
            ps82.setFetchSize(l.intValue());
        }
        List<NewProduct> npl = new ArrayList<NewProduct>();
        // Date currDate= System.getCurrDate
        Calendar cal = Calendar.getInstance(java.util.TimeZone.getDefault());

        Long cod = input.getCutOffDuration();
        long time = 0;
        if (cod != null) {
            time = cal.getTimeInMillis() - cod.longValue();
        } else {
            time = cal.getTimeInMillis();
        }
        Date date = new Date(time);
        Format formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String s = formatter.format(date);


        ps82.setString(1, input.getSubjectString());
        ps82.setString(2, s);

        ResultSet rs = ps82.executeQuery();
        while (rs.next()) {
            long i_id = rs.getLong(1);
            String i_title = rs.getString(2);
            String a_fname = rs.getString(3);
            String a_lname = rs.getString(4);
            NewProduct np = new NewProduct();
            np.setItemId(i_id);
            np.setItemTitle(i_title);
            np.setAuthorLName(a_lname);
            np.setAuthorFName(a_fname);
            npl.add(np);

        }
        if (rs != null) {
            rs.close();
        }
        npo.setNewProductsList(npl);
        ps82.close();
        return (npo);
    }
}
